/**
 * PROJECT: LogReceiver
 * UNIT:    LogReceiver.java
 * 
 *			-- ABSTRACT --
 *
 * Java demo program shows use of class KK_Library in package brendes.jna version 1.8
 * -> brendes.jna-1.8.jar
 * 
 * It is required that another K+K program is running which is connected to a K+K counter.
 * The K+K program must have started an local TCP server.
 * Please enter IP address and port of this TCP server into below String server.
 * 
 * LogReceiver connects to TCP server addressed in below String server and receives
 * log entries in an endless loop. 
 * Loop is left, if no more data is received from TCP server or a max. receive count is reached.
 * 
 * 
 * 			-- CONTACT --
 * 
 * Brendes Datentechnik, Loryn Brendes, info@brendes.de
 * 
 * 
 * 			-- HISTORY --
 *
 * date	      who	what
 * 2018-11-23 lb	uses KK-Library version 1.3
 * 2021-05-28 lb    uses KK-Library version 19.0.2
 * 2121-12-20 lb    uses KK-Library version 19.2.0
 * 2122-10-09 lb    uses KK-Library version 19.3.0
 */

import java.io.File;

import brendes.jna.KK_Library;
import brendes.jna.KK_Library.DebugLogType;
import brendes.jna.KK_Library.KK_Result;

public class LogReceiver {
	
	public static final String APP_NAME	= "LogReceiver";
	public static final String APP_VERSION = "1.3 2022-10-09";
	

	public static void main(String[] args) {
	
		System.out.println(APP_NAME+" version "+ APP_VERSION);
		//
		// TODO:
		// Please enter IP address and port of TCP server
		//
		String server = "127.0.0.1:48896";

		try {
			// Access to native library kk_fx80e via class KK_Library
			KK_Library kkLib = new KK_Library();
			System.out.println("KK_Library native lib version: "+kkLib.getDLLVersion());
			// source id
			int id = kkLib.createMultiSource();
		
			KK_Result result;
		
			// appDataPath
			String workingDir = System.getProperty("user.dir");
			int index = workingDir.indexOf(APP_NAME);
			String appDataPath = ".";
			if (index > 0)
				appDataPath = workingDir.substring(0, index-1);
			appDataPath = appDataPath+File.separator+"FXE_debug"+File.separator+APP_NAME;
			System.out.println("appDataPath: "+appDataPath);
			// output path
			result = kkLib.setOutputPath(id, appDataPath);
			if (result.resultCode != KK_Library.ErrorCode.KK_No_Err)
				System.out.println("OutputPath failed: "+result.data);
			else
				System.out.println("OutputPath: no error");
			// check it
			result = kkLib.getOutputPath(id);
			System.out.println("getOutputPath returns "+result.data);
			
			// Debug log
			// KK_ParamError should not occur
			kkLib.setDebugFlags(id, true, true);
			kkLib.setDebugLogLimit(id, DebugLogType.logCreateNew, 0);
			result = kkLib.setDebug(id, true, null);
			if (result.resultCode != KK_Library.ErrorCode.KK_No_Err)
				System.out.println("KK_Debug: "+result.data);
			else
				System.out.println("KK_Debug: no error");
			
			// Open TCP log
			//
			// TODO:
			// select log entry mode by commenting not used lines
			//
			//String mode;
			//mode = KK_Library.LOG_MODE_PHASE;
			//mode = KK_Library.LOG_MODE_FREQ;
			//mode = KK_Library.LOG_MODE_PHASEDIFF;
			
			// with prepended UTC timestamp
			String timeFormat = "YYYYMMDD HH:NN:SS.ZZZ";
			//result = kkLib.openTcpLogTime(id, server, mode, timeFormat);
			
			// without timestamp
			//result = kkLib.openTcpLog(id, server, mode);
			
			// with number and timestamp
			int logType = 0;
			result = kkLib.openTcpLogType(id, server, logType, timeFormat);
			
			if (result.resultCode != KK_Library.ErrorCode.KK_No_Err) {
				System.out.println("openTcpLog failed: "+result.data);
				return;
			} else {
				System.out.println("openTcpLog ok, server="+server);
			}
			
			// Receive logs
			boolean end = false;
			int emptyStringCounter = 0;
			int receivedCounter = 0;
			// TODO set limit
			int limitReceiveCounter = 25;
			while (!end) {
				result = kkLib.getTcpLog(id);
				if (result.resultCode == KK_Library.ErrorCode.KK_Err_BufferOverflow)
					System.out.println("getTcpLog reports overflow error");
				else if (result.resultCode == KK_Library.ErrorCode.KK_Err_ServerDown) {
					System.out.println("getTcpLog reports no connection to TCP server");
					end = true;
				}
				if (result.data == null) {
					// no data available
					emptyStringCounter++;
					if (emptyStringCounter > 50) {
						end = true;	// no more data, break loop
						System.out.println("no more data -> stop");
					}
					// wait 100 ms
					try {
						Thread.sleep(100);
					} catch (InterruptedException e) {
						e.printStackTrace();
						end = true;
					}
				} else {
					System.out.println(result.data);
					emptyStringCounter = 0;	// reset counter
					receivedCounter++;
					if (receivedCounter % 10 == 0) {
						System.out.println("send data to TCP server");
						String tcpData = "TCP Data after "+receivedCounter+" received logs";
						result = kkLib.tcpAppData(id, tcpData);
						if (result.resultCode == KK_Library.ErrorCode.KK_No_Err)
							System.out.println("send data to TCP server: "+result.data);
						else
							System.out.println("send data to TCP server failed: "+result.data);
					}
					else if (receivedCounter > limitReceiveCounter) {
						System.out.println("more than "+limitReceiveCounter+" logs received -> stop");
						end = true;						
					}
				}
			}
			
			// Close TCP log
			System.out.println("closeTcpLog");
			kkLib.closeTcpLog(id);
		}
		catch (Exception e) {
			System.out.println("Exception "+e.getClass().getName()+": "+e.getMessage());
		}
	}
}